package com.example.kosherappstore

import android.app.Activity
import android.content.ActivityNotFoundException
import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.content.pm.ApplicationInfo
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.provider.Settings
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.widget.Button
import android.widget.EditText
import android.widget.ListView
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.FileProvider
import com.example.kosherappstore.adapters.AppListAdapter
import com.example.kosherappstore.data.AppInfo
import com.example.kosherappstore.utils.Constants
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import java.io.BufferedReader
import java.io.DataOutputStream
import java.io.File
import java.io.InputStreamReader

class MainActivity : AppCompatActivity() {

    private lateinit var appsListView: ListView
    private lateinit var emptyListTextView: TextView
    private lateinit var settingsButtonMain: Button

    private var appList = mutableListOf<AppInfo>()
    private lateinit var adapter: AppListAdapter
    private lateinit var sharedPreferences: SharedPreferences
    private val gson = Gson()

    private var pendingInstallAppInfo: AppInfo? = null

    private val TAG = "MainActivity"

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        Log.d(TAG, "onCreate called")

        appsListView = findViewById(R.id.apps_list_view)
        emptyListTextView = findViewById(R.id.empty_list_text_view)
        settingsButtonMain = findViewById(R.id.settings_button_main)

        sharedPreferences = getSharedPreferences(Constants.PREFS_NAME, Context.MODE_PRIVATE)

        adapter = AppListAdapter(
            this,
            appList,
            onInstallClick = { appInfo, _ ->
                Log.d(TAG, "Install clicked for ${appInfo.name}")
                handleInstallApp(appInfo)
            },
            onUninstallClick = { appInfo, _ ->
                Log.d(TAG, "Uninstall clicked for ${appInfo.name}")
                handleUninstallApp(appInfo)
            }
        )
        appsListView.adapter = adapter
        appsListView.emptyView = emptyListTextView

        settingsButtonMain.setOnClickListener {
            Log.d(TAG, "Settings button clicked")
            val storedPassword = sharedPreferences.getString(Constants.KEY_SETTINGS_PASSWORD, Constants.DEFAULT_SETTINGS_PASSWORD)

            // אם הסיסמה השמורה היא null (לא אמור לקרות עם ברירת מחדל),
            // או ריקה (המשתמש מחק את הסיסמה),
            // או שווה לברירת המחדל (ובהנחה שברירת המחדל היא "סיסמה לא פעילה" מבחינת המשתמש)
            // אז פותחים ישירות.
            // אם yossi6547 צריכה להיות סיסמה פעילה, שנה את הלוגיקה הזו.
            if (storedPassword.isNullOrEmpty() || storedPassword == Constants.DEFAULT_SETTINGS_PASSWORD) {
                Log.d(TAG, "No active user-defined password set (or using default). Opening settings directly.")
                openSettingsActivity()
            } else {
                Log.d(TAG, "Active password found. Showing password dialog.")
                showEnterPasswordDialog(storedPassword)
            }
        }
    }

    private fun openSettingsActivity() {
        val intent = Intent(this, SettingsActivity::class.java)
        startActivityForResult(intent, Constants.REQUEST_CODE_SETTINGS)
        // Log.d(TAG, "SettingsActivity opened.") // אפשר להוסיף לוג אם רוצים
    }

    private fun showEnterPasswordDialog(correctPassword: String) {
        val dialogView = LayoutInflater.from(this).inflate(R.layout.dialog_enter_password, null)
        val passwordInputEditText = dialogView.findViewById<EditText>(R.id.password_input_edit_text)

        AlertDialog.Builder(this)
            .setTitle(R.string.dialog_title_enter_password)
            .setView(dialogView)
            .setPositiveButton(R.string.ok_button_text) { dialog, _ ->
                val enteredPassword = passwordInputEditText.text.toString()
                if (enteredPassword == correctPassword) {
                    Log.i(TAG, "Password correct. Opening settings.")
                    openSettingsActivity()
                } else {
                    Log.w(TAG, "Password incorrect. Entered: '$enteredPassword', Expected (hash/actual): '$correctPassword'")
                    Toast.makeText(this, getString(R.string.wrong_password_message), Toast.LENGTH_SHORT).show()
                }
                // אין צורך ב-dialog.dismiss() כי הדיאלוג נסגר אוטומטית בלחיצה על כפתור חיובי/שלילי
            }
            .setNegativeButton(R.string.cancel_button_text, null) // null listener פשוט סוגר את הדיאלוג
            .show()
    }

    override fun onResume() {
        super.onResume()
        Log.d(TAG, "onResume called. Refreshing app list.")
        loadAppsFromPreferencesAndUpdateAdapter()
    }

    private fun handleInstallApp(appInfo: AppInfo) {
        val installMode = sharedPreferences.getString(Constants.KEY_INSTALLATION_MODE, Constants.INSTALL_MODE_NORMAL)
        Log.d(TAG, "Installation mode for ${appInfo.name}: $installMode")

        if (installMode == Constants.INSTALL_MODE_ROOT) {
            val rootEnabledInSettings = sharedPreferences.getBoolean(Constants.KEY_ROOT_INSTALL_ENABLED, true)
            if (rootEnabledInSettings) {
                Log.i(TAG, "Root installation requested for ${appInfo.name}")
                handleRootInstall(appInfo)
                return
            } else {
                Log.w(TAG, "Root installation is selected in mode, but disabled by switch. Proceeding with normal install.")
                Toast.makeText(this, "Root installation disabled. Attempting normal install.", Toast.LENGTH_LONG).show()
                installApkNormally(appInfo)
                return
            }
        }
        installApkNormally(appInfo)
    }

    private fun handleRootInstall(appInfo: AppInfo) {
        val rootMethod = sharedPreferences.getString(Constants.KEY_ROOT_INSTALL_METHOD, Constants.ROOT_METHOD_PM_INSTALL)
        Log.d(TAG, "Root installation method for ${appInfo.name}: $rootMethod")

        val apkFileInternal = File(filesDir, appInfo.apkFileName)
        if (!apkFileInternal.exists() || !apkFileInternal.canRead()) {
            Log.e(TAG, "APK file not found or not readable for root install: ${apkFileInternal.absolutePath}")
            Toast.makeText(this, getString(R.string.app_not_found_for_install) + " (Root)", Toast.LENGTH_LONG).show()
            return
        }

        when (rootMethod) {
            Constants.ROOT_METHOD_PM_INSTALL -> {
                Log.i(TAG, "Executing root install using 'pm install' method for ${apkFileInternal.absolutePath}")
                val command = "pm install -r -g \"${apkFileInternal.absolutePath}\""
                executeRootCommand(listOf(command), "Installing ${appInfo.name} via pm install...", requiresRebootNotification = false) { success ->
                    if (success) {
                        val appToUpdate = appList.find { it.packageName == appInfo.packageName }
                        appToUpdate?.let {
                            if (it.wasInstalledAsSystemByStore) {
                                it.wasInstalledAsSystemByStore = false
                                saveAppListToPreferences()
                                Log.i(TAG, "App ${it.packageName} marked as NOT system app after pm install.")
                            }
                        }
                    }
                }
            }
            Constants.ROOT_METHOD_COPY_TO_SYSTEM -> {
                Log.i(TAG, "Executing root install using 'copy to system' method for ${appInfo.name}")
                val systemInstallPathChosen = sharedPreferences.getString(Constants.KEY_COPY_TO_SYSTEM_PATH, Constants.SYSTEM_PATH_APP)
                val targetDirNameInSystem = appInfo.packageName
                val targetApkNameInSystem = "base.apk"

                executeCopyToSystemRootCommands(
                    apkFileInternal.absolutePath,
                    systemInstallPathChosen!!,
                    targetDirNameInSystem,
                    targetApkNameInSystem,
                    appInfo
                )
            }
            else -> {
                Log.e(TAG, "Unknown root installation method: $rootMethod")
                Toast.makeText(this, "Unknown root installation method.", Toast.LENGTH_LONG).show()
            }
        }
    }

    private fun executeCopyToSystemRootCommands(
        sourceApkPath: String,
        systemBaseDir: String,
        targetDirNameInSystem: String,
        targetApkNameInSystem: String,
        appInfoToUpdate: AppInfo
    ) {
        val targetDirPath = "$systemBaseDir/$targetDirNameInSystem"
        val targetApkPath = "$targetDirPath/$targetApkNameInSystem"
        Log.d(TAG, "Copy to system details: source='$sourceApkPath', targetDir='$targetDirPath', targetApk='$targetApkPath'")

        val commands = mutableListOf<String>()
        commands.add("mount -o rw,remount /system")
        commands.add("mkdir -p \"$targetDirPath\"")
        commands.add("cp -f \"$sourceApkPath\" \"$targetApkPath\"")
        commands.add("chmod 755 \"$targetDirPath\"")
        commands.add("chmod 644 \"$targetApkPath\"")
        commands.add("chown 0.0 \"$targetDirPath\"")
        commands.add("chown 0.0 \"$targetApkPath\"")
        commands.add("mount -o ro,remount /system")

        executeRootCommand(commands, "Copying ${appInfoToUpdate.name} to $systemBaseDir...", true) { success ->
            if (success) {
                val appInList = appList.find { it.packageName == appInfoToUpdate.packageName }
                appInList?.let {
                    it.wasInstalledAsSystemByStore = true
                    saveAppListToPreferences()
                    Log.i(TAG, "App ${it.packageName} marked as system app by store.")
                }
            }
        }
    }

    private fun executeRootCommand(
        commands: List<String>,
        progressMessage: String,
        requiresRebootNotification: Boolean = false,
        onResult: ((success: Boolean) -> Unit)? = null
    ) {
        Toast.makeText(this, progressMessage, Toast.LENGTH_SHORT).show()
        Log.d(TAG, "Executing list of root commands. Requires reboot: $requiresRebootNotification")
        commands.forEach { Log.d("$TAG-RootCmd", it) }

        Thread {
            var process: Process? = null
            var os: DataOutputStream? = null
            var reader: BufferedReader? = null
            var errorReader: BufferedReader? = null
            val output = StringBuilder()
            val errorOutput = StringBuilder()
            var operationSucceeded = false

            try {
                process = Runtime.getRuntime().exec("su")
                os = DataOutputStream(process!!.outputStream)
                reader = BufferedReader(InputStreamReader(process!!.inputStream, "UTF-8"))
                errorReader = BufferedReader(InputStreamReader(process!!.errorStream, "UTF-8"))

                for (command in commands) {
                    Log.d(TAG, "Executing: $command")
                    os.writeBytes("$command\n")
                    os.flush()
                }
                os.writeBytes("exit\n")
                os.flush()

                var line: String?
                while (reader.readLine().also { line = it } != null) { output.append(line).append("\n"); Log.d("$TAG-RootOutput", line!!) }
                while (errorReader.readLine().also { line = it } != null) { errorOutput.append(line).append("\n"); Log.e("$TAG-RootError", line!!) }

                val exitCode = process.waitFor()
                Log.d(TAG, "Root command sequence exited with code: $exitCode")

                val isPmInstallCommand = commands.any { it.startsWith("pm install") }
                if (exitCode == 0) {
                    if (isPmInstallCommand && output.toString().contains("Success", ignoreCase = true)) {
                        operationSucceeded = true
                    } else if (!isPmInstallCommand) {
                        operationSucceeded = true
                        if (errorOutput.toString().trim().isNotEmpty()) {
                            Log.w(TAG, "Root sequence (non-pm) exited with 0, but had error stream output: $errorOutput")
                        }
                    }
                }

                if(operationSucceeded) {
                    Log.i(TAG, "Root command sequence executed successfully. Output: $output")
                    runOnUiThread {
                        var message = getString(R.string.root_operation_successful)
                        if (requiresRebootNotification && !commands.contains("reboot")) {
                            message += "\nDevice reboot may be required."
                            showRebootDialog()
                        }
                        Toast.makeText(this, message, Toast.LENGTH_LONG).show()
                    }
                } else {
                    Log.e(TAG, "Root command sequence failed. Exit code: $exitCode, Output: $output, Error: $errorOutput")
                    val errorMessageToShow = if (errorOutput.isNotBlank()) errorOutput.toString() else if (output.isNotBlank()) output.toString() else "Unknown error (Code: $exitCode)"
                    runOnUiThread { Toast.makeText(this, getString(R.string.root_operation_failed, errorMessageToShow.trim()), Toast.LENGTH_LONG).show() }
                }

            } catch (e: SecurityException) {
                operationSucceeded = false
                Log.e(TAG, "Root access denied or not available.", e)
                runOnUiThread { Toast.makeText(this, getString(R.string.root_not_available_or_denied), Toast.LENGTH_LONG).show() }
            } catch (e: Exception) {
                operationSucceeded = false
                Log.e(TAG, "Error executing root command sequence", e)
                runOnUiThread { Toast.makeText(this, getString(R.string.root_operation_failed, e.localizedMessage ?: "Unknown error"), Toast.LENGTH_LONG).show() }
            } finally {
                try { os?.close(); reader?.close(); errorReader?.close(); process?.destroy() }
                catch (e: Exception) { Log.e(TAG, "Error closing streams for root command sequence", e) }

                onResult?.invoke(operationSucceeded)

                runOnUiThread {
                    Log.d(TAG, "Root command sequence finished. Refreshing list.")
                    loadAppsFromPreferencesAndUpdateAdapter()
                }
            }
        }.start()
    }

    private fun handleUninstallApp(appInfo: AppInfo) {
        val installMode = sharedPreferences.getString(Constants.KEY_INSTALLATION_MODE, Constants.INSTALL_MODE_NORMAL)
        var isActuallySystemApp = false
        try {
            isActuallySystemApp = (packageManager.getApplicationInfo(appInfo.packageName, 0).flags and ApplicationInfo.FLAG_SYSTEM) != 0
        } catch (e: PackageManager.NameNotFoundException) {
            Log.w(TAG, "Package ${appInfo.packageName} not found during uninstall check, proceeding with normal uninstall logic.")
        }

        Log.d(TAG, "Uninstall requested for ${appInfo.packageName}. By store as system: ${appInfo.wasInstalledAsSystemByStore}, Actually system: $isActuallySystemApp")

        if (appInfo.wasInstalledAsSystemByStore &&
            installMode == Constants.INSTALL_MODE_ROOT &&
            sharedPreferences.getBoolean(Constants.KEY_ROOT_INSTALL_ENABLED, true)) {

            val systemInstallPath = sharedPreferences.getString(Constants.KEY_COPY_TO_SYSTEM_PATH, Constants.SYSTEM_PATH_APP)
            val targetDirNameInSystem = appInfo.packageName
            executeRemoveSystemAppRootCommands(systemInstallPath!!, targetDirNameInSystem, appInfo)
        } else {
            Log.d(TAG, "Proceeding with normal uninstallation for ${appInfo.packageName}")
            val uninstallIntent = Intent(Intent.ACTION_DELETE)
            uninstallIntent.data = Uri.parse("package:${appInfo.packageName}")
            uninstallIntent.putExtra(Intent.EXTRA_RETURN_RESULT, true)
            try {
                Toast.makeText(this, getString(R.string.starting_uninstallation_for, appInfo.name), Toast.LENGTH_SHORT).show()
                startActivityForResult(uninstallIntent, Constants.REQUEST_CODE_UNINSTALL_APP)
            } catch (e: Exception) {
                Log.e(TAG, "Error starting normal uninstallation for ${appInfo.packageName}", e)
                Toast.makeText(this, "Error initiating uninstallation: ${e.localizedMessage}", Toast.LENGTH_LONG).show()
            }
        }
    }

    private fun executeRemoveSystemAppRootCommands(
        systemBaseDir: String,
        targetDirNameInSystem: String,
        appInfoToUpdate: AppInfo
    ) {
        val targetDirPath = "$systemBaseDir/$targetDirNameInSystem"
        Log.d(TAG, "Attempting to remove system app from: $targetDirPath")

        val commands = mutableListOf<String>()
        commands.add("mount -o rw,remount /system")
        commands.add("rm -rf \"$targetDirPath\"")
        commands.add("mount -o ro,remount /system")
        val pmUninstallCommand = "pm uninstall ${appInfoToUpdate.packageName}"

        executeRootCommand(commands, "Removing ${appInfoToUpdate.name} files from system...", true) { fileRemovalSuccess ->
            if (fileRemovalSuccess) {
                Log.i(TAG, "System files for ${appInfoToUpdate.packageName} removed. Now attempting pm uninstall.")
                executeRootCommand(listOf(pmUninstallCommand), "Finalizing removal of ${appInfoToUpdate.name}...", false) { pmUninstallSuccess ->
                    val appInList = appList.find { it.packageName == appInfoToUpdate.packageName }
                    appInList?.let {
                        it.wasInstalledAsSystemByStore = false
                        saveAppListToPreferences()
                        Log.i(TAG, "App ${it.packageName} marked as NOT system app by store after removal attempt.")
                    }
                    if (!pmUninstallSuccess) {
                        Log.w(TAG, "pm uninstall for ${appInfoToUpdate.packageName} might have failed, but files were removed.")
                    }
                }
            } else {
                Log.e(TAG, "Failed to remove system files for ${appInfoToUpdate.packageName}.")
            }
        }
    }

    private fun saveAppListToPreferences() {
        val jsonApps = gson.toJson(appList)
        sharedPreferences.edit().putString(Constants.KEY_APPS_LIST, jsonApps).apply() // Using apply() for SharedPreferences
        Log.d(TAG, "App list saved to SharedPreferences.")
    }

    private fun showRebootDialog() {
        AlertDialog.Builder(this)
            .setTitle("Reboot Required")
            .setMessage("The operation might require a device reboot to take full effect. Reboot now?")
            .setPositiveButton("Reboot Now") { _, _ ->
                Log.i(TAG, "User chose to reboot.")
                executeRootCommand(listOf("reboot"), "Rebooting device...")
            }
            .setNegativeButton("Later", null)
            .setCancelable(false)
            .show()
    }

    private fun installApkNormally(appInfo: AppInfo) {
        Log.d(TAG, "Attempting normal installation for ${appInfo.name}")
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            if (!packageManager.canRequestPackageInstalls()) {
                Log.w(TAG, "REQUEST_INSTALL_PACKAGES permission not granted.")
                pendingInstallAppInfo = appInfo
                showRequestInstallPermissionDialog()
                return
            }
        }
        Log.d(TAG, "REQUEST_INSTALL_PACKAGES permission is granted or not required.")
        proceedWithNormalInstallation(appInfo)
    }

    private fun proceedWithNormalInstallation(appInfo: AppInfo) {
        val apkFile = File(filesDir, appInfo.apkFileName)
        Log.d(TAG, "APK file path for ${appInfo.name}: ${apkFile.absolutePath}")

        if (!apkFile.exists()) {
            Log.e(TAG, "APK file not found: ${apkFile.absolutePath}")
            Toast.makeText(this, getString(R.string.app_not_found_for_install), Toast.LENGTH_LONG).show()
            return
        }

        val apkUri: Uri
        try {
            apkUri = FileProvider.getUriForFile(this, "${applicationContext.packageName}.provider", apkFile)
            Log.d(TAG, "FileProvider URI for ${apkFile.name}: $apkUri")
        } catch (e: Exception) {
            Log.e(TAG, "Error getting URI from FileProvider", e)
            Toast.makeText(this, "Error creating URI for APK file.", Toast.LENGTH_LONG).show()
            return
        }

        val installIntent = Intent(Intent.ACTION_VIEW)
        installIntent.setDataAndType(apkUri, "application/vnd.android.package-archive")
        installIntent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
        installIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)

        try {
            Log.i(TAG, "Starting package installer for ${appInfo.name}")
            Toast.makeText(this, getString(R.string.starting_installation_for, appInfo.name), Toast.LENGTH_SHORT).show()
            startActivity(installIntent)
        } catch (e: ActivityNotFoundException) {
            Log.e(TAG, "Package installer not found.", e)
            Toast.makeText(this, "Could not find package installer.", Toast.LENGTH_LONG).show()
        } catch (e: Exception) {
            Log.e(TAG, "Error starting package installer", e)
            Toast.makeText(this, "Error starting installation: ${e.localizedMessage}", Toast.LENGTH_LONG).show()
        }
    }

    private fun showRequestInstallPermissionDialog() {
        AlertDialog.Builder(this)
            .setTitle(R.string.permission_required_title)
            .setMessage(R.string.install_package_permission_request_message)
            .setPositiveButton(R.string.go_to_settings_button) { _, _ ->
                val intent = Intent(Settings.ACTION_MANAGE_UNKNOWN_APP_SOURCES)
                intent.data = Uri.parse("package:$packageName")
                try {
                    startActivityForResult(intent, Constants.REQUEST_CODE_INSTALL_PACKAGES)
                } catch (e: Exception) {
                    Log.e(TAG, "Error opening unknown app sources settings", e)
                    Toast.makeText(this, "Could not open settings screen.", Toast.LENGTH_SHORT).show()
                }
            }
            .setNegativeButton(android.R.string.cancel, null)
            .show()
    }

    private fun loadAppsFromPreferencesAndUpdateAdapter() {
        Log.d(TAG, "Loading apps from preferences for adapter update...")
        val jsonApps = sharedPreferences.getString(Constants.KEY_APPS_LIST, null)
        appList.clear()
        if (jsonApps != null && jsonApps.isNotEmpty() && jsonApps != "null") {
            try {
                val type = object : TypeToken<MutableList<AppInfo>>() {}.type
                val loadedApps: MutableList<AppInfo> = gson.fromJson(jsonApps, type)
                if (loadedApps != null) {
                    appList.addAll(loadedApps)
                    Log.d(TAG, "Apps successfully deserialized. New appList count: ${appList.size}")
                } else {
                    Log.e(TAG, "gson.fromJson returned null for JSON.")
                }
            } catch (e: Exception) {
                Log.e(TAG, "Error parsing apps from preferences", e)
            }
        } else {
            Log.d(TAG, "No valid apps JSON found in SharedPreferences or list is empty.")
        }
        adapter.notifyDataSetChanged()
        checkEmptyViewVisibility()
        Log.d(TAG, "Adapter notified. Adapter count: ${adapter.count}. AppList size: ${appList.size}")
    }

    private fun checkEmptyViewVisibility() {
        if (appList.isEmpty()) {
            emptyListTextView.visibility = View.VISIBLE
            appsListView.visibility = View.GONE
        } else {
            emptyListTextView.visibility = View.GONE
            appsListView.visibility = View.VISIBLE
        }
    }

    @Deprecated("Deprecated in Java")
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        Log.d(TAG, "onActivityResult: requestCode=$requestCode, resultCode=$resultCode")
        if (requestCode == Constants.REQUEST_CODE_SETTINGS) {
            // onResume will handle list refresh
        } else if (requestCode == Constants.REQUEST_CODE_INSTALL_PACKAGES) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                if (packageManager.canRequestPackageInstalls()) {
                    Log.i(TAG, "REQUEST_INSTALL_PACKAGES permission granted after returning from settings.")
                    pendingInstallAppInfo?.let {
                        Log.d(TAG, "Proceeding with installation for pending app: ${it.name}")
                        proceedWithNormalInstallation(it)
                        pendingInstallAppInfo = null
                    }
                } else {
                    Log.w(TAG, "REQUEST_INSTALL_PACKAGES permission still NOT granted.")
                    Toast.makeText(this, "Permission to install packages is still required.", Toast.LENGTH_LONG).show()
                }
            }
        } else if (requestCode == Constants.REQUEST_CODE_UNINSTALL_APP) {
            if (resultCode == Activity.RESULT_OK) {
                Toast.makeText(this, getString(R.string.uninstallation_succeeded), Toast.LENGTH_SHORT).show()
                Log.i(TAG, "Uninstallation succeeded for an app.")
            } else {
                Log.w(TAG, "Uninstallation result was not RESULT_OK. ResultCode: $resultCode")
            }
            // onResume will refresh the list
        }
    }
}