package com.example.kosherappstore

import android.content.Context
import android.content.SharedPreferences
import android.os.Bundle
import android.util.Log
import android.view.MenuItem
import android.view.View
import android.widget.ListView
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AlertDialog // השתמש ב-AlertDialog של androidx
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.edit // Import for SharedPreferences KTX extension
import com.example.kosherappstore.adapters.ManageAppsAdapter
import com.example.kosherappstore.data.AppInfo
import com.example.kosherappstore.utils.Constants
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import java.io.File

class ManageStoreAppsActivity : AppCompatActivity() { // אין צורך ב-android.app.Activity כאן

    private lateinit var appsListView: ListView
    private lateinit var emptyListTextView: TextView
    private var storeAppList = mutableListOf<AppInfo>()
    private lateinit var adapter: ManageAppsAdapter
    private lateinit var sharedPreferences: SharedPreferences
    private val gson = Gson()
    private val TAG = "ManageStoreAppsActivity"

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_manage_store_apps)

        // אתחול ה-Views. אם ה-IDs ב-XML שונים, שנה אותם כאן או ב-XML.
        appsListView = findViewById(R.id.manage_apps_list_view)
        emptyListTextView = findViewById(R.id.manage_empty_list_text_view)

        sharedPreferences = getSharedPreferences(Constants.PREFS_NAME, Context.MODE_PRIVATE)

        adapter = ManageAppsAdapter(this, storeAppList) { appInfoToConfirmDelete, position ->
            showConfirmDeleteDialog(appInfoToConfirmDelete, position)
        }
        appsListView.adapter = adapter
        appsListView.emptyView = emptyListTextView // מקשר את ה-TextView ל-ListView

        Log.d(TAG, "onCreate: Activity created.")
    }

    override fun onResume() {
        super.onResume()
        Log.d(TAG, "onResume: Loading apps for management.")
        loadAppsForManagement()
    }

    private fun loadAppsForManagement() {
        val jsonApps = sharedPreferences.getString(Constants.KEY_APPS_LIST, null)
        storeAppList.clear()
        if (jsonApps != null) {
            try {
                val type = object : TypeToken<MutableList<AppInfo>>() {}.type
                val loadedApps: MutableList<AppInfo> = gson.fromJson(jsonApps, type)
                storeAppList.addAll(loadedApps)
                Log.d(TAG, "Loaded ${storeAppList.size} apps for management.")
            } catch (e: Exception) {
                Log.e(TAG, "Error loading apps for management from SharedPreferences", e)
                Toast.makeText(this, getString(R.string.error_loading_apps_data), Toast.LENGTH_SHORT).show()
            }
        } else {
            Log.d(TAG, "No apps found in SharedPreferences for management.")
        }
        adapter.notifyDataSetChanged()
        checkEmptyView()
    }

    private fun checkEmptyView() {
        if (storeAppList.isEmpty()) {
            emptyListTextView.visibility = View.VISIBLE
            appsListView.visibility = View.GONE
        } else {
            emptyListTextView.visibility = View.GONE
            appsListView.visibility = View.VISIBLE
        }
    }

    private fun showConfirmDeleteDialog(appInfo: AppInfo, position: Int) {
        AlertDialog.Builder(this) // אין צורך ב- androidx.appcompat.app.AlertDialog כאן, this.AlertDialog יספיק אם ה-import נכון
            .setTitle(R.string.confirm_delete_from_store_title)
            .setMessage(getString(R.string.confirm_delete_from_store_message, appInfo.name, appInfo.packageName))
            .setPositiveButton(R.string.delete_button_text) { dialog, _ ->
                deleteAppFromStore(appInfo, position)
                dialog.dismiss()
            }
            .setNegativeButton(R.string.cancel_button_text, null)
            .show()
    }

    private fun deleteAppFromStore(appInfo: AppInfo, position: Int) {
        Log.d(TAG, "Attempting to delete app from store: ${appInfo.packageName}")

        if (position >= 0 && position < storeAppList.size) {
            storeAppList.removeAt(position)
        } else {
            val removed = storeAppList.remove(appInfo) // Try to remove by object if position is invalid
            if (!removed) {
                Log.w(TAG, "Could not remove app by position or object: ${appInfo.packageName}")
                // Consider finding by package name as a last resort if needed
                // storeAppList.removeAll { it.packageName == appInfo.packageName }
            }
        }

        // שימוש ב-KTX extension function עבור SharedPreferences.edit
        sharedPreferences.edit { // this: SharedPreferences.Editor
            val jsonApps = gson.toJson(storeAppList)
            putString(Constants.KEY_APPS_LIST, jsonApps)
            // apply() נקרא אוטומטית בסוף הבלוק של edit {}
            Log.d(TAG, "Updated app list saved to SharedPreferences. New count: ${storeAppList.size}")
        }


        if (appInfo.apkFileName.isNotEmpty()) {
            val apkFile = File(filesDir, appInfo.apkFileName)
            if (apkFile.exists()) {
                if (apkFile.delete()) {
                    Log.i(TAG, "APK file deleted: ${apkFile.absolutePath}")
                } else {
                    Log.e(TAG, "Failed to delete APK file: ${apkFile.absolutePath}")
                    Toast.makeText(this, getString(R.string.error_deleting_app_files, "APK"), Toast.LENGTH_SHORT).show()
                }
            } else {
                Log.w(TAG, "APK file not found for deletion: ${apkFile.absolutePath}")
            }
        }

        if (!appInfo.imageFileName.isNullOrEmpty()) {
            val imageFile = File(filesDir, appInfo.imageFileName)
            if (imageFile.exists()) {
                if (imageFile.delete()) {
                    Log.i(TAG, "Image file deleted: ${imageFile.absolutePath}")
                } else {
                    Log.e(TAG, "Failed to delete image file: ${imageFile.absolutePath}")
                    Toast.makeText(this, getString(R.string.error_deleting_app_files, "Image"), Toast.LENGTH_SHORT).show()
                }
            } else {
                Log.w(TAG, "Image file not found for deletion: ${imageFile.absolutePath}")
            }
        }

        Toast.makeText(this, getString(R.string.app_deleted_from_store_successfully, appInfo.name), Toast.LENGTH_SHORT).show()
        adapter.notifyDataSetChanged()
        checkEmptyView()
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            onBackPressedDispatcher.onBackPressed()
            return true
        }
        return super.onOptionsItemSelected(item)
    }
}