package com.example.kosherappstore

import android.Manifest
import android.app.Activity
import android.content.ActivityNotFoundException
import android.content.ContentResolver
import android.content.Context
import android.content.DialogInterface
import android.content.Intent
import android.content.SharedPreferences
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.Environment
import android.provider.OpenableColumns
import android.text.TextUtils
import android.util.Log
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.View
import android.widget.Button
import android.widget.EditText
import android.widget.LinearLayout
import android.widget.RadioButton
import android.widget.RadioGroup
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AlertDialog // Correct import for AppCompat AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.widget.SwitchCompat
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import androidx.core.content.edit
import com.example.kosherappstore.data.AppInfo
import com.example.kosherappstore.utils.Constants
import com.google.gson.Gson
import com.google.gson.JsonSyntaxException
import com.google.gson.reflect.TypeToken
import java.io.File
import java.io.FileInputStream
import java.io.FileOutputStream
import java.io.FileReader
import java.io.IOException
import java.io.InputStream
import java.io.OutputStream
import java.text.SimpleDateFormat
import java.util.Date
import java.util.Locale
import java.util.zip.ZipEntry
import java.util.zip.ZipInputStream
import java.util.zip.ZipOutputStream


class SettingsActivity : AppCompatActivity() {

    private lateinit var sharedPreferences: SharedPreferences
    private val gson = Gson()
    private var appList = mutableListOf<AppInfo>()

    // Views
    private lateinit var settingsPasswordEditText: EditText
    private lateinit var savePasswordButton: Button
    private lateinit var addNewAppButton: Button
    private lateinit var installModeRadioGroup: RadioGroup
    private lateinit var normalInstallRadioButton: RadioButton
    private lateinit var rootInstallRadioButton: RadioButton
    private lateinit var rootOptionsLayout: LinearLayout
    private lateinit var enableRootInstallSwitch: SwitchCompat
    private lateinit var rootMethodRadioGroup: RadioGroup
    private lateinit var pmInstallRadioButton: RadioButton
    private lateinit var copyToSystemRadioButton: RadioButton
    private lateinit var copyToSystemPathLayout: LinearLayout
    private lateinit var systemPathRadioGroup: RadioGroup
    private lateinit var systemAppRadioButton: RadioButton
    private lateinit var systemPrivAppRadioButton: RadioButton
    private lateinit var saveInstallationSettingsButton: Button
    private lateinit var manageStoreAppsButton: Button
    private lateinit var exportStoreButton: Button
    private lateinit var importStoreButton: Button

    // Temp vars for Add App Dialog
    private var tempSelectedApkUri: Uri? = null
    private var tempSelectedImageUri: Uri? = null
    private var tempSelectedApkFileName: String? = null
    private var tempSelectedImageFileName: String? = null
    private var dialogSelectedApkTextView: TextView? = null
    private var dialogSelectedImageTextView: TextView? = null

    private val logTag = "SettingsActivity"

    data class StoreManifest(
        val preferences: Map<String, Any?>,
        val appList: List<AppInfo>
    )

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_settings)
        Log.d(logTag, "onCreate: Activity starting.")

        sharedPreferences = getSharedPreferences(Constants.PREFS_NAME, MODE_PRIVATE) // Context.MODE_PRIVATE
        Log.d(logTag, "onCreate: SharedPreferences initialized.")
        loadAppsFromPreferences()

        initViews()
        setupListeners()
        loadInstallationSettings()
        setupInstallationOptionsListeners()

        Log.d(logTag, "onCreate finished successfully.")
    }

    private fun initViews() {
        settingsPasswordEditText = findViewById(R.id.settings_password_edit_text)
        savePasswordButton = findViewById(R.id.save_settings_password_button)
        addNewAppButton = findViewById(R.id.add_new_app_button)
        installModeRadioGroup = findViewById(R.id.install_mode_radio_group)
        normalInstallRadioButton = findViewById(R.id.normal_install_radio_button)
        rootInstallRadioButton = findViewById(R.id.root_install_radio_button)
        rootOptionsLayout = findViewById(R.id.root_options_layout)
        enableRootInstallSwitch = findViewById(R.id.enable_root_install_switch)
        rootMethodRadioGroup = findViewById(R.id.root_method_radio_group)
        pmInstallRadioButton = findViewById(R.id.pm_install_radio_button)
        copyToSystemRadioButton = findViewById(R.id.copy_to_system_radio_button)
        copyToSystemPathLayout = findViewById(R.id.copy_to_system_path_layout)
        systemPathRadioGroup = findViewById(R.id.system_path_radio_group)
        systemAppRadioButton = findViewById(R.id.system_app_radio_button)
        systemPrivAppRadioButton = findViewById(R.id.system_priv_app_radio_button)
        saveInstallationSettingsButton = findViewById(R.id.save_installation_settings_button)
        manageStoreAppsButton = findViewById(R.id.manage_store_apps_button)
        exportStoreButton = findViewById(R.id.export_store_button)
        importStoreButton = findViewById(R.id.import_store_button)
        Log.d(logTag, "All views initialized.")
    }

    private fun setupListeners() {
        savePasswordButton.setOnClickListener {
            val newPassword = settingsPasswordEditText.text.toString()
            if (newPassword.isNotEmpty()) {
                sharedPreferences.edit { putString(Constants.KEY_SETTINGS_PASSWORD, newPassword) }
                Toast.makeText(this, "הסיסמה נשמרה בהצלחה.", Toast.LENGTH_SHORT).show()
                settingsPasswordEditText.text.clear()
            } else { Toast.makeText(this, "שדה הסיסמה אינו יכול להיות ריק לשמירה.", Toast.LENGTH_SHORT).show() }
        }
        addNewAppButton.setOnClickListener { showAddAppDialog() }
        saveInstallationSettingsButton.setOnClickListener {
            saveInstallationSettings()
            Toast.makeText(this, "הגדרות ההתקנה נשמרו.", Toast.LENGTH_SHORT).show()
        }
        manageStoreAppsButton.setOnClickListener {
            startActivity(Intent(this, ManageStoreAppsActivity::class.java))
        }
        exportStoreButton.setOnClickListener { checkAndRequestWritePermission() }
        importStoreButton.setOnClickListener {
            Log.d(logTag, "Import Store button clicked")
            AlertDialog.Builder(this) // Using AppCompat AlertDialog (via import)
                .setTitle("אישור ייבוא")
                .setMessage("ייבוא נתונים מקובץ ZIP יחליף את כל הגדרות החנות ורשימת האפליקציות הקיימות. האם אתה בטוח שברצונך להמשיך?")
                .setPositiveButton(android.R.string.ok) { _: DialogInterface, _: Int -> openFilePickerForZip() }
                .setNegativeButton(android.R.string.cancel, null)
                .show()
        }
        Log.d(logTag, "All main listeners set up.")
    }

    // <editor-fold desc="onOptionsItemSelected, load/saveAppsToPreferences, Add App Dialog and related">
    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            onBackPressedDispatcher.onBackPressed(); return true
        }
        return super.onOptionsItemSelected(item)
    }
    private fun loadAppsFromPreferences() {
        val jsonApps = sharedPreferences.getString(Constants.KEY_APPS_LIST, null); appList.clear()
        if (jsonApps != null) {
            try { val type = object : TypeToken<MutableList<AppInfo>>() {}.type; val loaded: MutableList<AppInfo> = gson.fromJson(jsonApps, type) ?: mutableListOf(); appList.addAll(loaded) }
            catch (e: Exception) { Log.e(logTag, "Error loading apps from preferences", e) }
        }
    }
    private fun saveAppsToPreferences() {
        try { sharedPreferences.edit { putString(Constants.KEY_APPS_LIST, gson.toJson(appList)) } }
        catch (e: Exception) { Log.e(logTag, "Error saveAppsToPreferences", e); Toast.makeText(this, "Error saving app list", Toast.LENGTH_SHORT).show() }
    }
    private fun showAddAppDialog() {
        tempSelectedApkUri = null; tempSelectedImageUri = null; tempSelectedApkFileName = null; tempSelectedImageFileName = null
        val v = LayoutInflater.from(this).inflate(R.layout.dialog_add_app, null)
        val nameEt = v.findViewById<EditText>(R.id.dialog_app_name_edit_text)
        val descEt = v.findViewById<EditText>(R.id.dialog_app_description_edit_text)
        val pkgEt = v.findViewById<EditText>(R.id.dialog_app_package_name_edit_text)
        dialogSelectedApkTextView = v.findViewById(R.id.dialog_selected_apk_text_view)
        dialogSelectedImageTextView = v.findViewById(R.id.dialog_selected_image_text_view)
        dialogSelectedApkTextView?.text = "לא נבחר קובץ APK"
        dialogSelectedImageTextView?.text = "לא נבחר אייקון"
        v.findViewById<Button>(R.id.dialog_select_apk_button).setOnClickListener { openFilePicker("application/vnd.android.package-archive", Constants.REQUEST_CODE_PICK_APK) }
        v.findViewById<Button>(R.id.dialog_select_image_button).setOnClickListener { openFilePicker("image/*", Constants.REQUEST_CODE_PICK_IMAGE) }
        AlertDialog.Builder(this) // Using AppCompat AlertDialog (via import)
            .setView(v)
            .setTitle("הוספת אפליקציה חדשה")
            .setPositiveButton("הוסף אפליקציה", null)
            .setNegativeButton("ביטול", null)
            .create().apply { setOnShowListener { d -> (d as AlertDialog).getButton(DialogInterface.BUTTON_POSITIVE).setOnClickListener { // Changed to DialogInterface
                val appName = nameEt.text.toString().trim(); val appDesc = descEt.text.toString().trim(); val appPkg = pkgEt.text.toString().trim()
                if (TextUtils.isEmpty(appName) || TextUtils.isEmpty(appPkg) || tempSelectedApkUri == null) { Toast.makeText(this@SettingsActivity, "חובה למלא שם אפליקציה, שם חבילה ולבחור קובץ APK.", Toast.LENGTH_LONG).show(); return@setOnClickListener }
                if (tempSelectedApkFileName.isNullOrEmpty()) { Toast.makeText(this@SettingsActivity, "שגיאה: לא זוהה שם קובץ ה-APK.", Toast.LENGTH_LONG).show(); return@setOnClickListener }
                val intApkName = "apk_${System.currentTimeMillis()}_${sanitizeFileName(tempSelectedApkFileName!!)}"; var intImgName: String? = null
                if (!saveUriToFile(tempSelectedApkUri!!, intApkName)) { Toast.makeText(this@SettingsActivity, "שגיאה בשמירת קובץ ה-APK.", Toast.LENGTH_LONG).show(); return@setOnClickListener }
                tempSelectedImageFileName?.takeIf { it.isNotEmpty() && tempSelectedImageUri != null }?.let { imgFName ->
                    intImgName = "img_${System.currentTimeMillis()}_${sanitizeFileName(imgFName)}"
                    if (!saveUriToFile(tempSelectedImageUri!!, intImgName!!)) { intImgName = null; Toast.makeText(this@SettingsActivity, "אזהרה: שגיאה בשמירת קובץ התמונה...", Toast.LENGTH_SHORT).show() }
                }
                appList.add(AppInfo(appName, appDesc, intApkName, intImgName, appPkg)); saveAppsToPreferences()
                Toast.makeText(this@SettingsActivity, "האפליקציה '$appName' נוספה לחנות בהצלחה.", Toast.LENGTH_SHORT).show(); setResult(Activity.RESULT_OK); d.dismiss()
            }}}
            .show()
    }
    private fun sanitizeFileName(fileName: String): String = fileName.replace(Regex("[^a-zA-Z0-9._-]"), "_").replace(" ", "_")
    @Suppress("DEPRECATION")
    private fun openFilePicker(mimeType: String, requestCode: Int) {
        val i = Intent(Intent.ACTION_GET_CONTENT).apply { type = mimeType; addCategory(Intent.CATEGORY_OPENABLE) }
        try { startActivityForResult(Intent.createChooser(i, "בחר קובץ"), requestCode) }
        catch (ex: ActivityNotFoundException) { Log.e(logTag, "No file manager", ex); Toast.makeText(this, "לא נמצאה אפליקציה לבחירת קבצים.", Toast.LENGTH_SHORT).show() }
    }
    override fun onRequestPermissionsResult(requestCode: Int, p: Array<out String>, grants: IntArray) {
        super.onRequestPermissionsResult(requestCode, p, grants)
        if (requestCode == Constants.REQUEST_CODE_WRITE_EXTERNAL_STORAGE && grants.isNotEmpty() && grants[0] == PackageManager.PERMISSION_GRANTED) {
            initiateExportProcess(null)
        } else if (requestCode == Constants.REQUEST_CODE_WRITE_EXTERNAL_STORAGE) { Toast.makeText(this, "Write permission required", Toast.LENGTH_LONG).show() }
    }
    @Suppress("DEPRECATION")
    @Deprecated("Deprecated in Java")
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        Log.d(logTag, "onActivityResult: req=$requestCode, res=$resultCode")
        if (resultCode == Activity.RESULT_OK && data?.data != null) {
            when (requestCode) {
                Constants.REQUEST_CODE_PICK_APK -> {
                    tempSelectedApkUri = data.data!!; tempSelectedApkFileName = getFileNameFromUri(data.data!!)
                    dialogSelectedApkTextView?.text = "APK נבחר: ${tempSelectedApkFileName ?: "unknown"}"
                }
                Constants.REQUEST_CODE_PICK_IMAGE -> {
                    tempSelectedImageUri = data.data!!; tempSelectedImageFileName = getFileNameFromUri(data.data!!)
                    dialogSelectedImageTextView?.text = "אייקון נבחר: ${tempSelectedImageFileName ?: "unknown"}"
                }
                Constants.REQUEST_CODE_CREATE_DOCUMENT_SAF -> data.data?.let { initiateExportProcess(it) } ?: Toast.makeText(this, "Failed to get save location.", Toast.LENGTH_SHORT).show()
                Constants.REQUEST_CODE_OPEN_DOCUMENT_SAF -> data.data?.let { initiateImportProcess(it) } ?: Toast.makeText(this, "Failed to get selected ZIP.", Toast.LENGTH_SHORT).show()
            }
        } else { Log.w(logTag, "onActivityResult: Request $requestCode did not return RESULT_OK or data is null") }
    }
    private fun getFileNameFromUri(uri: Uri): String? {
        var fileName: String? = null
        if (ContentResolver.SCHEME_CONTENT == uri.scheme) { // No qualifier needed for ContentResolver
            try { contentResolver.query(uri, null, null, null, null)?.use { c -> if (c.moveToFirst()) c.getString(c.getColumnIndexOrThrow(OpenableColumns.DISPLAY_NAME)).also { fileName = it } } }
            catch (e: Exception) { Log.e(logTag, "Error getting file name from content URI", e) }
        }
        if (fileName == null) { uri.path?.let { path -> val cut = path.lastIndexOf('/'); if (cut != -1) fileName = path.substring(cut + 1) } }
        return fileName?.takeIf { it.isNotEmpty() }
    }
    private fun saveUriToFile(uri: Uri, outputFileName: String): Boolean {
        try { contentResolver.openInputStream(uri)?.use { iStream -> FileOutputStream(File(filesDir, outputFileName)).use { oStream -> iStream.copyTo(oStream); return true } } }
        catch (e: IOException) { Log.e(logTag, "Error saving URI to file '$outputFileName'", e); Toast.makeText(this, "Error copying file", Toast.LENGTH_LONG).show() }
        return false
    }
    // </editor-fold>

    // <editor-fold desc="Installation Options Logic">
    private fun setupInstallationOptionsListeners() {
        installModeRadioGroup.setOnCheckedChangeListener { _, id -> updateRootOptionsVisibilityUI(id == R.id.root_install_radio_button) }
        enableRootInstallSwitch.setOnCheckedChangeListener { _, isChecked -> updateRootMethodAndPathEnabledStateUI(rootInstallRadioButton.isChecked && isChecked) }
        rootMethodRadioGroup.setOnCheckedChangeListener { _, id -> updateCopyToSystemPathVisibilityUI(rootInstallRadioButton.isChecked && enableRootInstallSwitch.isChecked && id == R.id.copy_to_system_radio_button) }
    }
    private fun updateRootOptionsVisibilityUI(showRoot: Boolean) {
        rootOptionsLayout.visibility = if (showRoot) View.VISIBLE else View.GONE
        if (showRoot) updateRootMethodAndPathEnabledStateUI(enableRootInstallSwitch.isChecked)
        else { updateRootMethodAndPathEnabledStateUI(false); copyToSystemPathLayout.visibility = View.GONE }
    }
    private fun updateRootMethodAndPathEnabledStateUI(enableSub: Boolean) {
        for (i in 0 until rootMethodRadioGroup.childCount) { rootMethodRadioGroup.getChildAt(i).isEnabled = enableSub }
        updateCopyToSystemPathVisibilityUI(enableSub && copyToSystemRadioButton.isChecked)
    }
    private fun updateCopyToSystemPathVisibilityUI(showPath: Boolean) {
        copyToSystemPathLayout.visibility = if (showPath) View.VISIBLE else View.GONE
        for (i in 0 until systemPathRadioGroup.childCount) { systemPathRadioGroup.getChildAt(i).isEnabled = showPath }
    }
    private fun loadInstallationSettings() {
        sharedPreferences.apply {
            if (getString(Constants.KEY_INSTALLATION_MODE, Constants.INSTALL_MODE_NORMAL) == Constants.INSTALL_MODE_ROOT) rootInstallRadioButton.isChecked = true else normalInstallRadioButton.isChecked = true
            enableRootInstallSwitch.isChecked = getBoolean(Constants.KEY_ROOT_INSTALL_ENABLED, true)
            if (getString(Constants.KEY_ROOT_INSTALL_METHOD, Constants.ROOT_METHOD_PM_INSTALL) == Constants.ROOT_METHOD_COPY_TO_SYSTEM) copyToSystemRadioButton.isChecked = true else pmInstallRadioButton.isChecked = true
            if (getString(Constants.KEY_COPY_TO_SYSTEM_PATH, Constants.SYSTEM_PATH_APP) == Constants.SYSTEM_PATH_PRIV_APP) systemPrivAppRadioButton.isChecked = true else systemAppRadioButton.isChecked = true
        }
        updateRootOptionsVisibilityUI(rootInstallRadioButton.isChecked)
    }
    private fun saveInstallationSettings() {
        sharedPreferences.edit {
            putString(Constants.KEY_INSTALLATION_MODE, if (rootInstallRadioButton.isChecked) Constants.INSTALL_MODE_ROOT else Constants.INSTALL_MODE_NORMAL)
            putBoolean(Constants.KEY_ROOT_INSTALL_ENABLED, enableRootInstallSwitch.isChecked)
            putString(Constants.KEY_ROOT_INSTALL_METHOD, if (copyToSystemRadioButton.isChecked) Constants.ROOT_METHOD_COPY_TO_SYSTEM else Constants.ROOT_METHOD_PM_INSTALL)
            putString(Constants.KEY_COPY_TO_SYSTEM_PATH, if (systemPrivAppRadioButton.isChecked) Constants.SYSTEM_PATH_PRIV_APP else Constants.SYSTEM_PATH_APP)
        }
    }
    // </editor-fold>

    // <editor-fold desc="Export Logic">
    private fun checkAndRequestWritePermission() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) openSafForSave()
        else if (ContextCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE) == PackageManager.PERMISSION_GRANTED) initiateExportProcess(null)
        else ActivityCompat.requestPermissions(this, arrayOf(Manifest.permission.WRITE_EXTERNAL_STORAGE), Constants.REQUEST_CODE_WRITE_EXTERNAL_STORAGE)
    }
    @Suppress("DEPRECATION")
    private fun openSafForSave() {
        val timeStamp = SimpleDateFormat("yyyyMMdd_HHmmss", Locale.getDefault()).format(Date())
        val fileName = "${Constants.ZIP_DEFAULT_NAME_PREFIX}${timeStamp}.zip"
        val intent = Intent(Intent.ACTION_CREATE_DOCUMENT).apply { addCategory(Intent.CATEGORY_OPENABLE); type = "application/zip"; putExtra(Intent.EXTRA_TITLE, fileName) }
        try { startActivityForResult(intent, Constants.REQUEST_CODE_CREATE_DOCUMENT_SAF) }
        catch (e: Exception) { Log.e(logTag, "SAF Error", e); Toast.makeText(this, "File saver error", Toast.LENGTH_LONG).show() }
    }
    private fun initiateExportProcess(outputFileUri: Uri?) {
        Toast.makeText(this, "יוצר קובץ גיבוי ZIP...", Toast.LENGTH_LONG).show() // Hardcoded
        Thread {
            var success = false; var finalZipPathMsg: String? = null; var errorMsg: String? = null
            try {
                val prefs = sharedPreferences.all.filterKeys { it != Constants.KEY_APPS_LIST }
                val manifest = mapOf("preferences" to prefs, "appList" to appList)
                val manifestJson = gson.toJson(manifest)
                val manifestFile = File(cacheDir, Constants.MANIFEST_FILE_NAME).apply { writeText(manifestJson) }
                val outStream: OutputStream = if (outputFileUri != null && Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
                    contentResolver.openOutputStream(outputFileUri) ?: throw IOException("SAF stream null")
                } else {
                    val dlDir = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS); dlDir.mkdirs()
                    val zipF = File(dlDir, "${Constants.ZIP_DEFAULT_NAME_PREFIX}${SimpleDateFormat("yyyyMMdd_HHmmss", Locale.getDefault()).format(Date())}.zip")
                    finalZipPathMsg = zipF.absolutePath; FileOutputStream(zipF)
                }
                ZipOutputStream(outStream).use { zos ->
                    zos.putNextEntry(ZipEntry(Constants.MANIFEST_FILE_NAME)); FileInputStream(manifestFile).use { it.copyTo(zos) }; zos.closeEntry(); manifestFile.delete()
                    appList.forEach { app ->
                        if (app.apkFileName.isNotEmpty()) File(filesDir, app.apkFileName).takeIf {it.exists()}?.let {
                            zos.putNextEntry(ZipEntry("${Constants.APKS_FOLDER_NAME}/${app.apkFileName}")); FileInputStream(it).use {fis->fis.copyTo(zos)}; zos.closeEntry()
                        }
                        app.imageFileName?.takeIf{it.isNotEmpty()}?.let {imgName -> File(filesDir, imgName).takeIf{it.exists()}?.let {
                            zos.putNextEntry(ZipEntry("${Constants.IMAGES_FOLDER_NAME}/$imgName")); FileInputStream(it).use{fis->fis.copyTo(zos)}; zos.closeEntry()
                        }}
                    }
                }; success = true
            } catch (e: Exception) { Log.e(logTag, "Export error", e); errorMsg = e.localizedMessage }
            runOnUiThread {
                val builder = AlertDialog.Builder(this) // Using AppCompat AlertDialog
                if (success) {
                    val pathMsg = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q && outputFileUri != null) "File saved via picker." else finalZipPathMsg ?: "location."
                    builder.setTitle("ייצוא הושלם").setMessage("החנות יוצאה בהצלחה לקובץ: $pathMsg")
                } else {
                    builder.setTitle("ייצוא נכשל").setMessage("אירעה שגיאה במהלך ייצוא החנות: ${errorMsg ?: "Unknown"}")
                }
                builder.setPositiveButton(android.R.string.ok, null).show()
            }
        }.start()
    }
    // </editor-fold>

    // <editor-fold desc="Import Logic">
    @Suppress("DEPRECATION")
    private fun openFilePickerForZip() {
        val intent = Intent(Intent.ACTION_GET_CONTENT).apply { type = "application/zip"; addCategory(Intent.CATEGORY_OPENABLE) }
        try { startActivityForResult(Intent.createChooser(intent, "בחר קובץ גיבוי ZIP..."), Constants.REQUEST_CODE_OPEN_DOCUMENT_SAF) } // Hardcoded
        catch (e: ActivityNotFoundException) { Toast.makeText(this, "לא נמצאה אפליקציה לבחירת קבצים.", Toast.LENGTH_SHORT).show() }
        catch (e: Exception) { Toast.makeText(this, "Picker error: ${e.message}", Toast.LENGTH_LONG).show() }
    }
    private fun initiateImportProcess(zipUri: Uri) {
        Toast.makeText(this, "מעבד קובץ ZIP...", Toast.LENGTH_LONG).show() // Hardcoded
        Log.i(logTag, "Initiating import process from URI: $zipUri")
        Thread {
            var success = false; var errorMsg: String? = null
            val tmpZip = File(cacheDir, "import.zip"); val tmpExtract = File(cacheDir, "import_extract")
            try {
                contentResolver.openInputStream(zipUri)?.use { i -> FileOutputStream(tmpZip).use { o -> i.copyTo(o) } } ?: throw IOException("InputStream ZIP null")
                if (tmpExtract.exists()) tmpExtract.deleteRecursively(); tmpExtract.mkdirs()
                ZipInputStream(FileInputStream(tmpZip)).use { zis -> var entry: ZipEntry? = zis.nextEntry
                    while (entry != null) { val newF = File(tmpExtract, entry.name)
                        if (!newF.canonicalPath.startsWith(tmpExtract.canonicalPath + File.separator)) throw SecurityException("Zip slip")
                        if (entry.isDirectory) { if (!newF.isDirectory && !newF.mkdirs()) throw IOException("Mkdir fail") }
                        else { val parent = newF.parentFile; if (parent!=null&&!parent.isDirectory&&!parent.mkdirs()) throw IOException("Mkdir parent fail"); FileOutputStream(newF).use { it.write(zis.readBytes()) } }
                        zis.closeEntry(); entry = zis.nextEntry
                    }
                }
                success = processImportedData(tmpExtract)
            } catch (e: Exception) { Log.e(logTag, "Import error", e); errorMsg = e.localizedMessage }
            finally { if(tmpZip.exists()) tmpZip.delete() }
            runOnUiThread {
                val builder = AlertDialog.Builder(this) // Using AppCompat AlertDialog
                if (success) {
                    builder.setTitle("ייבוא הושלם")
                        .setMessage("נתוני החנות יובאו בהצלחה. מומלץ להפעיל מחדש את האפליקציה.")
                } else {
                    builder.setTitle("ייבוא נכשל")
                        .setMessage("אירעה שגיאה במהלך ייבוא הנתונים: ${errorMsg ?: "Unknown"}")
                    if (tmpExtract.exists()) tmpExtract.deleteRecursively()
                }
                builder.setPositiveButton(android.R.string.ok, null).show()
                if (success) { loadAppsFromPreferences(); loadInstallationSettings() }
            }
        }.start()
    }
    private fun processImportedData(extractDir: File): Boolean {
        val manifestFile = File(extractDir, Constants.MANIFEST_FILE_NAME)
        if (!manifestFile.exists()) {
            Log.e(logTag, "${Constants.MANIFEST_FILE_NAME} not found");
            runOnUiThread{Toast.makeText(this, "מבנה קובץ ה-ZIP אינו תקין או חסר manifest.json.", Toast.LENGTH_LONG).show()}; return false // Hardcoded
        }
        try {
            val manifest = gson.fromJson(FileReader(manifestFile), StoreManifest::class.java)
            sharedPreferences.edit { clear()
                manifest.preferences.forEach { (k, v) -> when (v) {
                    is String -> putString(k, v); is Int -> putInt(k, v); is Long -> putLong(k, v)
                    is Float -> putFloat(k, v); is Boolean -> putBoolean(k, v)
                    is Double -> if (v == v.toLong().toDouble()) if (v <= Int.MAX_VALUE && v >= Int.MIN_VALUE) putInt(k, v.toInt()) else putLong(k, v.toLong()) else putFloat(k, v.toFloat())
                    else -> Log.w(logTag, "Unsupported pref type for $k: ${v?.javaClass?.simpleName}")
                }}
            }
            val importedApps = mutableListOf<AppInfo>()
            val apksExtDir = File(extractDir, Constants.APKS_FOLDER_NAME); val imgsExtDir = File(extractDir, Constants.IMAGES_FOLDER_NAME)
            manifest.appList.forEach { impApp -> var curApk = impApp.apkFileName; var curImg: String? = impApp.imageFileName
                if (impApp.apkFileName.isNotEmpty()) File(apksExtDir, impApp.apkFileName).takeIf{it.exists()}?.let { srcApk ->
                    val newIntApk = "apk_${System.currentTimeMillis()}_${sanitizeFileName(impApp.apkFileName)}"; srcApk.copyTo(File(filesDir, newIntApk), true); curApk = newIntApk
                } ?: Log.w(logTag, "APK ${impApp.apkFileName} not in ZIP for ${impApp.packageName}")
                impApp.imageFileName?.takeIf{it.isNotEmpty()}?.let { imgN -> File(imgsExtDir, imgN).takeIf{it.exists()}?.let { srcImg ->
                    val newIntImg = "img_${System.currentTimeMillis()}_${sanitizeFileName(imgN)}"; srcImg.copyTo(File(filesDir, newIntImg), true); curImg = newIntImg
                } ?: Log.w(logTag, "Image $imgN not in ZIP for ${impApp.packageName}") }
                importedApps.add(impApp.copy(apkFileName = curApk, imageFileName = curImg))
            }
            this.appList.clear(); this.appList.addAll(importedApps); saveAppsToPreferences()
            Log.i(logTag, "Imported ${this.appList.size} apps."); return true
        } catch (e: Exception) { Log.e(logTag, "Error processing manifest/files", e); runOnUiThread{Toast.makeText(this,"Processing error: ${e.message}",Toast.LENGTH_LONG).show()}; return false }
        finally { if (extractDir.exists()) extractDir.deleteRecursively() }
    }
    // </editor-fold>
}