package com.example.kosherappstore.adapters

import android.content.Context
import android.content.pm.PackageManager
import android.graphics.BitmapFactory
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import com.example.kosherappstore.R
import com.example.kosherappstore.data.AppInfo
import java.io.File

class AppListAdapter(
    context: Context,
    private val appList: List<AppInfo>, // השתמש ברשימה שהועברה, היא תתעדכן מבחוץ
    private val onInstallClick: (appInfo: AppInfo, position: Int) -> Unit,
    private val onUninstallClick: (appInfo: AppInfo, position: Int) -> Unit
) : ArrayAdapter<AppInfo>(context, 0, appList) { // חשוב שהרשימה כאן היא אותה רשימה ש-MainActivity מעדכנת

    private val inflater: LayoutInflater = LayoutInflater.from(context)
    private val packageManager: PackageManager = context.packageManager

    override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
        val view: View
        val holder: ViewHolder

        if (convertView == null) {
            view = inflater.inflate(R.layout.list_item_app, parent, false)
            holder = ViewHolder()
            holder.iconImageView = view.findViewById(R.id.app_icon_image_view)
            holder.nameTextView = view.findViewById(R.id.app_name_text_view)
            holder.descriptionTextView = view.findViewById(R.id.app_description_text_view)
            holder.installButton = view.findViewById(R.id.install_button)
            holder.uninstallButton = view.findViewById(R.id.uninstall_button)
            // holder.buttonsLayout = view.findViewById(R.id.buttons_layout) // אם קיים
            view.tag = holder
        } else {
            view = convertView
            holder = view.tag as ViewHolder
        }

        val appInfo = getItem(position)

        if (appInfo == null) {
            Log.e("AppListAdapter", "AppInfo at position $position is null")
            // הסתר את כל הפריט או הצג הודעת שגיאה בפריט
            holder.nameTextView.text = ""
            holder.descriptionTextView.text = ""
            holder.iconImageView.setImageDrawable(null)
            holder.installButton.visibility = View.GONE
            holder.uninstallButton.visibility = View.GONE
            return view
        }

        holder.nameTextView.text = appInfo.name
        holder.descriptionTextView.text = appInfo.description

        if (!appInfo.imageFileName.isNullOrEmpty()) {
            val imageFile = File(context.filesDir, appInfo.imageFileName)
            if (imageFile.exists()) {
                try {
                    val bitmap = BitmapFactory.decodeFile(imageFile.absolutePath)
                    holder.iconImageView.setImageBitmap(bitmap)
                } catch (e: Exception) {
                    holder.iconImageView.setImageResource(R.mipmap.ic_launcher)
                }
            } else {
                holder.iconImageView.setImageResource(R.mipmap.ic_launcher)
            }
        } else {
            holder.iconImageView.setImageResource(R.mipmap.ic_launcher)
        }

        val isAppInstalled = isPackageInstalled(appInfo.packageName)
        if (isAppInstalled) {
            holder.installButton.visibility = View.GONE
            holder.uninstallButton.visibility = View.VISIBLE
        } else {
            holder.installButton.visibility = View.VISIBLE
            holder.uninstallButton.visibility = View.GONE
        }

        holder.installButton.setOnClickListener {
            onInstallClick(appInfo, position)
        }
        holder.uninstallButton.setOnClickListener {
            onUninstallClick(appInfo, position)
        }

        // ניהול פוקוס D-Pad (כפי שהיה קודם)
        view.onFocusChangeListener = View.OnFocusChangeListener { _, hasFocus ->
            holder.installButton.isFocusable = hasFocus
            holder.uninstallButton.isFocusable = hasFocus
        }
        view.setOnClickListener {
            if (holder.installButton.visibility == View.VISIBLE) {
                holder.installButton.requestFocus()
            } else if (holder.uninstallButton.visibility == View.VISIBLE) {
                holder.uninstallButton.requestFocus()
            }
        }
        return view
    }

    private fun isPackageInstalled(packageName: String): Boolean {
        return try {
            // השימוש ב-GET_META_DATA הוא פשוט דגל כלשהו, אפשר גם 0.
            packageManager.getPackageInfo(packageName, PackageManager.GET_META_DATA)
            true
        } catch (e: PackageManager.NameNotFoundException) {
            false
        }
    }

    private class ViewHolder {
        lateinit var iconImageView: ImageView
        lateinit var nameTextView: TextView
        lateinit var descriptionTextView: TextView
        lateinit var installButton: Button
        lateinit var uninstallButton: Button
        // lateinit var buttonsLayout: LinearLayout
    }
}